@@ -13,23 +13,15 @@ import os
 from casatasks import casalog
 from casatools import table
 from dask import delayed
-from dask.distributed import Client
 from scipy.interpolate import CubicSpline
 from scipy.ndimage import gaussian_filter1d
 from scipy.interpolate import interp1d
 from meersolar.utils import *
 from meersolar.meerpipeline.flagging import single_ms_flag
 
-logging.getLogger("distributed").setLevel(logging.WARNING)
-
-
-try:
-    casalogfile = casalog.logfile()
-    os.system("rm -rf " + casalogfile)
-except BaseException:
-    pass
-
 datadir = get_datadir()
+logging.getLogger("distributed").setLevel(logging.ERROR)
+logging.getLogger("tornado.application").setLevel(logging.CRITICAL)
 
 
 def interpolate_nans(data):
@@ -171,7 +163,6 @@ def applysol(
     force_apply=False,
     soltype="basic",
     do_post_flag=False,
-    dry_run=False,
 ):
     """
     Apply flux calibrated and attenuation calibrated solutions
@@ -211,13 +202,6 @@ def applysol(
     limit_threads(n_threads=n_threads)
     from casatasks import applycal, flagdata, split, clearcal
 
-    if dry_run:
-        process = psutil.Process(os.getpid())
-        mem = round(process.memory_info().rss / 1024**3, 2)  # in GB
-        return mem
-    print(
-        f"Applying solutions on ms: {os.path.basename(msname)} from caltables: {','.join([os.path.basename(i) for i in gaintable])}\n"
-    )
     if soltype == "basic":
         check_file = "/.applied_sol"
     else:
@@ -228,12 +212,12 @@ def applysol(
             return 0
         else:
             if os.path.exists(msname + check_file) and force_apply:
-                with suppress_casa_output():
+                with suppress_output():
                     clearcal(vis=msname)
                     flagdata(vis=msname, mode="unflag", spw="0", flagbackup=False)
                 if os.path.exists(msname + ".flagversions"):
                     os.system("rm -rf " + msname + ".flagversions")
-            with suppress_casa_output():
+            with suppress_output():
                 applycal(
                     vis=msname,
                     gaintable=gaintable,
@@ -245,14 +229,13 @@ def applysol(
                     flagbackup=False,
                 )
         if overwrite_datacolumn:
-            print(f"Spliting corrected data for ms: {msname}.")
             outputvis = msname.split(".ms")[0] + "_cor.ms"
             if os.path.exists(outputvis):
                 os.system(f"rm -rf {outputvis}")
             touch_file_names = glob.glob(f"{msname}/.*")
             if len(touch_file_names) > 0:
                 touch_file_names = [os.path.basename(f) for f in touch_file_names]
-            with suppress_casa_output():
+            with suppress_output():
                 split(vis=msname, outputvis=outputvis, datacolumn="corrected")
             if os.path.exists(outputvis):
                 os.system(f"rm -rf {msname} {msname}.flagversions")
@@ -260,7 +243,6 @@ def applysol(
             for t in touch_file_names:
                 os.system(f"touch {msname}/{t}")
         if do_post_flag:
-            print(f"Post calibration flagging on: {msname}")
             if overwrite_datacolumn:
                 datacolumn = "data"
             else:
@@ -284,6 +266,7 @@ def applysol(
 
 def run_all_applysol(
     mslist,
+    dask_client,
     workdir,
     caldir,
     use_only_bandpass=False,
@@ -293,7 +276,6 @@ def run_all_applysol(
     do_post_flag=False,
     cpu_frac=0.8,
     mem_frac=0.8,
-    dask_addr=None,
 ):
     """
     Apply basic-calibration solutions on all target scans
@@ -302,6 +284,8 @@ def run_all_applysol(
     ----------
     mslist : str
         Measurement set list
+    dask_client : dask.client
+        Dask client
     workdir : str
         Working directory
     caldir : str
@@ -320,16 +304,19 @@ def run_all_applysol(
         CPU fraction to use
     mem_frac : float, optional
         Memory fraction to use
-    dask_addr : str, optional
-        Dask scheduler address
 
     Returns
     --------
     list
         Calibrated target scans
     """
-    start_time = time.time()
     try:
+        if cpu_frac > 0.8:
+            cpu_frac = 0.8
+        total_cpu = max(1, int(psutil.cpu_count() * cpu_frac))
+        if mem_frac > 0.8:
+            mem_frac = 0.8
+        total_mem = (psutil.virtual_memory().available * mem_frac) / (1024**3)  # In GB
         os.chdir(workdir)
         mslist = np.unique(mslist).tolist()
         parang = False
@@ -406,37 +393,12 @@ def run_all_applysol(
         mslist = filtered_mslist
         if len(mslist) == 0:
             print("No valid measurement set.")
-            print(f"Total time taken: {round(time.time()-start_time,2)}s")
             return 1
 
         ####################################
         # Applycal jobs
         ####################################
         print(f"Total ms list: {len(mslist)}")
-        ms_size_list = [get_column_size(ms) for ms in mslist]
-        mem_limit = max(ms_size_list)
-        if dask_addr is None:
-            dask_client, dask_cluster, n_jobs, n_threads, mem_limit, dask_dir = (
-                get_dask_client(
-                    len(mslist),
-                    dask_dir=workdir,
-                    cpu_frac=cpu_frac,
-                    mem_frac=mem_frac,
-                    min_mem_per_job=mem_limit,
-                )
-            )
-        else:
-            _, _, n_jobs, n_threads, mem_limit, dask_dir = get_dask_client(
-                len(mslist),
-                dask_dir=workdir,
-                cpu_frac=cpu_frac,
-                mem_frac=mem_frac,
-                min_mem_per_job=mem_limit,
-                only_cal=True,
-            )
-            os.system(f"rm -rf {dask_dir}")
-            dask_client = Client(address=dask_addr)
-        wait_for_dask_workers(dask_client,min_worker=1,timeout=60)
         tasks = []
         if scaled_bandpass_table != "" and os.path.exists(scaled_bandpass_table):
             bpass_table = scaled_bandpass_table
@@ -449,6 +411,16 @@ def run_all_applysol(
                 os.system(f"rm -rf {workdir}/.attcal")
             os.system(f"touch {workdir}/.noattcal")
             bpass_table = bandpass_table[0]
+        njobs = min(total_cpu, len(mslist))
+        n_threads = max(1, int(total_cpu / njobs))
+        mem_limit = total_mem / njobs
+
+        print("#################################")
+        print(f"Total dask worker: {njobs}")
+        print(f"CPU per worker: {n_threads}")
+        print(f"Memory per worker: {round(mem_limit,2)} GB")
+        print("#################################")
+
         for ms in mslist:
             interp = []
             final_gaintable = gaintable + [bpass_table]
@@ -473,27 +445,23 @@ def run_all_applysol(
                     force_apply=force_apply,
                 )
             )
-        futures = dask_client.compute(tasks)
-        results = list(dask_client.gather(futures))
-        dask_client.close()
-        if dask_addr is None:
-            dask_cluster.close()
-            os.system(f"rm -rf {dask_dir}")
+        print(
+            f"Applying solutions from caltables: {','.join([os.path.basename(i) for i in final_gaintable])}"
+        )
+        results = list(dask_client.gather(dask_client.compute(tasks)))
         if np.nansum(results) == 0:
             print("##################")
             print(
                 "Applying basic calibration solutions for target scans are done successfully."
             )
-            print("Total time taken : ", time.time() - start_time)
-            print("##################\n")
+            print("##################")
             return 0
         else:
             print("##################")
             print(
                 "Applying basic calibration solutions for target scans are not done successfully."
             )
-            print("Total time taken : ", time.time() - start_time)
-            print("##################\n")
+            print("##################")
             return 1
     except Exception as e:
         traceback.print_exc()
@@ -502,8 +470,7 @@ def run_all_applysol(
         print(
             "Applying basic calibration solutions for target scans are not done successfully."
         )
-        print("Total time taken : ", time.time() - start_time)
-        print("##################\n")
+        print("##################")
         return 1
 
 
@@ -521,7 +488,7 @@ def main(
     mem_frac=0.8,
     logfile=None,
     jobid=0,
-    dask_addr=None,
+    dask_client=None,
 ):
     """
     Apply calibration solutions to a list of measurement sets with optional post-flagging.
@@ -554,8 +521,8 @@ def main(
         Path to the logfile. If None, logging to file is disabled. Default is None.
     jobid : int, optional
         Identifier for tracking the job and saving PID. Default is 0.
-    dask_addr : str, optional
-        Dask scheduler address
+    dask_client : dask.client, optional
+        Dask client
 
     Returns
     -------
@@ -589,12 +556,22 @@ def main(
             )
     if observer == None:
         print("Remote link or jobname is blank. Not transmiting to remote logger.")
-    ############
+
+    dask_cluster = None
+    if dask_client is None:
+        dask_client, dask_cluster, dask_dir = get_local_dask_cluster(
+            2,
+            dask_dir=workdir,
+            cpu_frac=cpu_frac,
+            mem_frac=mem_frac,
+        )
+        nworker = max(2, int(psutil.cpu_count() * cpu_frac))
+        scale_worker_and_wait(dask_cluster, nworker)
 
     try:
-        print("\n###################################")
+        print("###################################")
         print("Starting applying solutions...")
-        print("###################################\n")
+        print("###################################")
 
         if caldir == "" or not os.path.exists(caldir):
             print("Provide existing caltable directory.")
@@ -603,6 +580,7 @@ def main(
             mslist = mslist.split(",")
             msg = run_all_applysol(
                 mslist,
+                dask_client,
                 workdir,
                 caldir,
                 use_only_bandpass=use_only_bandpass,
@@ -612,7 +590,6 @@ def main(
                 force_apply=force_apply,
                 cpu_frac=cpu_frac,
                 mem_frac=mem_frac,
-                dask_addr=dask_addr,
             )
     except Exception:
         traceback.print_exc()
@@ -623,6 +600,10 @@ def main(
             drop_cache(ms)
         drop_cache(workdir)
         clean_shutdown(observer)
+        if dask_cluster is not None:
+            dask_client.close()
+            dask_cluster.close()
+            os.system(f"rm -rf {dask_dir}")
     return msg
 
 
@@ -707,7 +688,7 @@ def cli():
 
     if len(sys.argv) == 1:
         parser.print_help(sys.stderr)
-        sys.exit(1)
+        return 1
 
     args = parser.parse_args()
 
