@@ -10,20 +10,12 @@ import time
 import glob
 import sys
 import os
-from casatasks import casalog
 from casatools import msmetadata
 from dask import delayed
-from dask.distributed import Client
 from meersolar.utils import *
 
-logging.getLogger("distributed").setLevel(logging.WARNING)
-
-try:
-    logfile = casalog.logfile()
-    os.system("rm -rf " + logfile)
-except BaseException:
-    pass
-
+logging.getLogger("distributed").setLevel(logging.ERROR)
+logging.getLogger("tornado.application").setLevel(logging.CRITICAL)
 datadir = get_datadir()
 
 
@@ -55,7 +47,6 @@ def perform_imaging(
     make_overlay=True,
     make_plots=True,
     logfile="imaging.log",
-    dry_run=False,
 ):
     """
     Perform spectropolarimetric snapshot imaging of a ms
@@ -124,10 +115,6 @@ def perform_imaging(
     """
     if os.path.exists(logfile):
         os.system(f"rm -rf {logfile}")
-    if dry_run:
-        process = psutil.Process(os.getpid())
-        usemem = round(process.memory_info().rss / 1024**3, 2)  # in GB
-        return usemem
     logger, logfile = create_logger(
         os.path.basename(logfile).split(".log")[0],
         logfile,
@@ -511,8 +498,8 @@ def perform_imaging(
 
 
 def run_all_imaging(
-    mslist=[],
-    mainlogger=None,
+    mslist,
+    dask_client,
     workdir="",
     outdir="",
     freqrange="",
@@ -537,7 +524,6 @@ def run_all_imaging(
     cpu_frac=0.8,
     mem_frac=0.8,
     logfile="imaging.log",
-    dask_addr=None,
 ):
     """
     Run spectropolarimetric snapshot imaging on a list of measurement sets
@@ -546,8 +532,8 @@ def run_all_imaging(
     ----------
     mslist : list
         Measurement set list
-    mainlogger : str
-        Python logger
+    dask_client : dask.client
+        Dask client
     workdir : str
         Work directory
     outdir : str
@@ -594,39 +580,13 @@ def run_all_imaging(
         CPU fraction to use
     mem_frac : float, optional
         Memory fraction to use
-    dask_addr : str, optional
-        Dask scheduler address
 
     Returns
     -------
     int
         Success message
     """
-    start_time = time.time()
     mslist = sorted(mslist)
-    observer = None
-    if mainlogger is None:
-        mainlog_file = workdir + "/logs/imaging_targets.log"
-        mainlogger, mainlog_file = create_logger(
-            os.path.basename(mainlog_file).split(".log")[0],
-            mainlog_file,
-            verbose=False,
-            get_print=True,
-        )
-        observer = None
-        if (
-            os.path.exists(f"{workdir}/jobname_password.npy")
-            and mainlog_file is not None
-        ):
-            time.sleep(5)
-            jobname, password = np.load(
-                f"{workdir}/jobname_password.npy", allow_pickle=True
-            )
-            if os.path.exists(mainlog_file):
-                print(f"Starting remote logger. Remote logger password: {password}")
-                observer = init_logger(
-                    "all_imaging", mainlog_file, jobname=jobname, password=password
-                )
     ###########################
     # WSClean container
     ###########################
@@ -641,7 +601,7 @@ def run_all_imaging(
             return 1
     try:
         if len(mslist) == 0:
-            mainlogger.error("Provide valid measurement set list.")
+            print("Provide valid measurement set list.")
             time.sleep(5)
             clean_shutdown(observer)
             return 1
@@ -668,9 +628,12 @@ def run_all_imaging(
             if checkcol:
                 filtered_mslist.append(ms)
             else:
-                mainlogger.warning(f"Issue in : {ms}")
+                print(f"Issue in : {ms}")
                 os.system(f"rm -rf {ms}")
         mslist = filtered_mslist
+        if len(mslist) == 0:
+            print("No valid measurement set is found.")
+            return 1
 
         #####################################
         # Determining spectro-temporal chunks
@@ -719,39 +682,29 @@ def run_all_imaging(
             total_fd += per_job_fd
         if total_fd <= 0:
             total_fd = 1
-        n_jobs = max(1, int(new_soft_limit / total_fd))
-        n_jobs = min(len(mslist), n_jobs)
+
+        if cpu_frac > 0.8:
+            cpu_frac = 0.8
+        total_cpu = max(1, int(psutil.cpu_count() * cpu_frac))
+        if mem_frac > 0.8:
+            mem_frac = 0.8
+        total_mem = (psutil.virtual_memory().available * mem_frac) / (1024**3)  # In GB
 
         #################################
-        # Dask client setup
+        # Determining per worker resource
         #################################
-        mem_limit = perform_imaging(dry_run=True)
-        if dask_addr is None:
-            dask_client, dask_cluster, n_jobs, n_threads, mem_limit, dask_dir = (
-                get_dask_client(
-                    n_jobs,
-                    dask_dir=workdir,
-                    cpu_frac=cpu_frac,
-                    mem_frac=mem_frac,
-                    min_cpu_per_job=3,
-                    min_mem_per_job=mem_limit,
-                )
-            )
-        else:
-            _, _, n_jobs, n_threads, mem_limit, dask_dir = (
-                get_dask_client(
-                    n_jobs,
-                    dask_dir=workdir,
-                    cpu_frac=cpu_frac,
-                    mem_frac=mem_frac,
-                    min_cpu_per_job=3,
-                    min_mem_per_job=mem_limit,
-                    only_cal=True,
-                )
-            )
-            dask_client=Client(address=dask_addr)
-            os.system(f"rm -rf {dask_dir}")
-        wait_for_dask_workers(dask_client,min_worker=1,timeout=60)
+        njobs = min(len(mslist), int(new_soft_limit / total_fd))
+        njobs = max(1, min(total_cpu, njobs))
+        n_threads = max(1, int(total_cpu / njobs))
+        mem_limit = total_mem / njobs
+
+        print("#################################")
+        print(f"Total dask worker: {njobs}")
+        print(f"CPU per worker: {n_threads}")
+        print(f"Memory per worker: {round(mem_limit,2)} GB")
+        print("#################################")
+        #########################################
+
         tasks = []
         for i in range(len(mslist)):
             ms = mslist[i]
@@ -790,9 +743,6 @@ def run_all_imaging(
                 + os.path.basename(ms).split(".ms")[0]
                 + ".log"
             )
-            mainlogger.info(
-                f"Starting imaging for ms : {ms}, Log file : {logfile}\n",
-            )
             tasks.append(
                 delayed(perform_imaging)(
                     msname=ms,
@@ -823,50 +773,38 @@ def run_all_imaging(
                     logfile=logfile,
                 )
             )
-        futures = dask_client.compute(tasks)
-        results = list(dask_client.gather(futures))
-        dask_client.close()
-        if dask_addr is None:
-            dask_cluster.close()
-            os.system(f"rm -rf {dask_dir}")
+        print(
+            f"Starting imaging for ms : {ms}, Log file : {logfile}",
+        )
+        results = list(dask_client.gather(dask_client.compute(tasks)))
         all_image_list = []
         all_imaged_ms_list = []
         for i in range(len(results)):
             r = results[i][1]
             if len(r) == 0:
-                mainlogger.info(
+                print(
                     f"Imaging failed for ms : {mslist[i]}",
                 )
             else:
                 image_list = r["image"]
                 if len(image_list) == 0:
-                    mainlogger.info(
+                    print(
                         f"No image is made for ms : {mslist[i]}",
                     )
                 else:
                     all_imaged_ms_list.append(mslist[i])
                     for image in image_list:
                         all_image_list.append(image)
-        mainlogger.info(
+        print(
             f"Numbers of input measurement sets : {len(mslist)}.",
         )
-        mainlogger.info(
+        print(
             f"Imaging successfully done for: {len(all_imaged_ms_list)} measurement sets.",
         )
-        mainlogger.info(f"Total images made: {len(all_image_list)}.")
-        mainlogger.info(
-            f"Total time taken: {round(time.time()-start_time,2)}s",
-        )
-        time.sleep(5)
-        clean_shutdown(observer)
+        print(f"Total images made: {len(all_image_list)}.")
         return 0
     except Exception as e:
         traceback.print_exc()
-        mainlogger.info(
-            f"Total time taken: {round(time.time()-start_time,2)}s",
-        )
-        time.sleep(5)
-        clean_shutdown(observer)
         return 1
 
 
@@ -895,8 +833,9 @@ def main(
     start_remote_log=False,
     cpu_frac=0.8,
     mem_frac=0.8,
+    logfile=None,
     jobid=0,
-    dask_addr=None,
+    dask_client=None,
 ):
     """
     Perform distributed spectropolarimetric snapshot imaging on multiple measurement sets.
@@ -951,10 +890,12 @@ def main(
         Fraction of total CPUs to use per task. Default is 0.8.
     mem_frac : float, optional
         Fraction of total system memory to use per task. Default is 0.8.
+    logfile : str, optional
+        Log file
     jobid : int, optional
         Unique job identifier for logging and PID tracking. Default is 0.
-    dask_addr : str, optional
-        Dask scheduler address
+    dask_client : dask.client, optional
+        Dask client
 
     Returns
     -------
@@ -975,16 +916,6 @@ def main(
         outdir = workdir
     os.makedirs(outdir, exist_ok=True)
 
-    os.makedirs(workdir + "/logs/", exist_ok=True)
-
-    mainlog_file = workdir + "/logs/imaging_targets.log"
-    mainlogger, mainlog_file = create_logger(
-        os.path.basename(mainlog_file).split(".log")[0],
-        mainlog_file,
-        get_print=True,
-        verbose=False,
-    )
-
     ############
     # Logger
     ############
@@ -992,28 +923,38 @@ def main(
     if (
         start_remote_log
         and os.path.exists(f"{workdir}/jobname_password.npy")
-        and mainlog_file is not None
+        and logfile is not None
     ):
         time.sleep(5)
         jobname, password = np.load(
             f"{workdir}/jobname_password.npy", allow_pickle=True
         )
-        if os.path.exists(mainlog_file):
+        if os.path.exists(logfile):
             observer = init_logger(
-                "all_imaging", mainlog_file, jobname=jobname, password=password
+                "all_imaging", logfile, jobname=jobname, password=password
             )
     if observer == None:
         print("Remote link or jobname is blank. Not transmiting to remote logger.")
-    ##########
+
+    dask_cluster = None
+    if dask_client is None:
+        dask_client, dask_cluster, dask_dir = get_local_dask_cluster(
+            2,
+            dask_dir=workdir,
+            cpu_frac=cpu_frac,
+            mem_frac=mem_frac,
+        )
+        nworker = max(2, int(psutil.cpu_count() * cpu_frac))
+        scale_worker_and_wait(dask_cluster, nworker)
 
     try:
         if len(mslist) == 0:
-            mainlogger.info("Please provide a valid measurement set list.")
+            print("Please provide a valid measurement set list.")
             msg = 1
         else:
             msg = run_all_imaging(
-                mslist=mslist,
-                mainlogger=mainlogger,
+                mslist,
+                dask_client,
                 workdir=workdir,
                 outdir=outdir,
                 freqrange=freqrange,
@@ -1036,7 +977,6 @@ def main(
                 saveres=saveres,
                 cpu_frac=cpu_frac,
                 mem_frac=mem_frac,
-                dask_addr=dask_addr,
             )
     except Exception:
         traceback.print_exc()
@@ -1048,6 +988,10 @@ def main(
         drop_cache(workdir)
         drop_cache(outdir)
         clean_shutdown(observer)
+        if dask_cluster is not None:
+            dask_client.close()
+            dask_cluster.close()
+            os.system(f"rm -rf {dask_dir}")
     return msg
 
 
@@ -1223,7 +1167,7 @@ def cli():
 
     if len(sys.argv) == 1:
         parser.print_help(sys.stderr)
-        sys.exit(1)
+        return 1
 
     args = parser.parse_args()
 
