@@ -9,21 +9,13 @@ import time
 import sys
 import os
 import copy
-from casatasks import casalog
 from casatools import msmetadata, table
 from dask import delayed
-from dask.distributed import Client
 from functools import partial
 from meersolar.utils import *
 
-logging.getLogger("distributed").setLevel(logging.WARNING)
-
-try:
-    casalogfile = casalog.logfile()
-    os.system("rm -rf " + casalogfile)
-except BaseException:
-    pass
-
+logging.getLogger("distributed").setLevel(logging.ERROR)
+logging.getLogger("tornado.application").setLevel(logging.CRITICAL)
 datadir = get_datadir()
 
 
@@ -48,7 +40,6 @@ def do_selfcal(
     solar_selfcal=True,
     ncpu=-1,
     mem=-1,
-    dry_run=False,
     logfile="selfcal.log",
 ):
     """
@@ -109,10 +100,6 @@ def do_selfcal(
     limit_threads(n_threads=ncpu)
     from casatasks import split, flagdata, flagmanager
 
-    if dry_run:
-        process = psutil.Process(os.getpid())
-        mem = round(process.memory_info().rss / 1024**3, 2)  # in GB
-        return mem
     sub_observer = None
     logger, logfile = create_logger(
         os.path.basename(logfile).split(".log")[0], logfile, verbose=False
@@ -144,7 +131,7 @@ def do_selfcal(
         ##############################
         # Restoring any previous flags
         ##############################
-        with suppress_casa_output():
+        with suppress_output():
             flags = flagmanager(vis=msname, mode="list")
         keys = flags.keys()
         for k in keys:
@@ -153,7 +140,7 @@ def do_selfcal(
             else:
                 version = flags[0]["name"]
                 try:
-                    with suppress_casa_output():
+                    with suppress_output():
                         flagmanager(vis=msname, mode="restore", versionname=version)
                         flagmanager(vis=msname, mode="delete", versionname=version)
                 except BaseException:
@@ -173,7 +160,7 @@ def do_selfcal(
         msmd.close()
         if hascor:
             logger.info(f"Spliting corrected data to ms : {selfcalms}")
-            with suppress_casa_output():
+            with suppress_output():
                 split(
                     vis=msname,
                     field=str(field),
@@ -183,7 +170,7 @@ def do_selfcal(
                 )
         else:
             logger.info(f"Spliting data to ms : {selfcalms}")
-            with suppress_casa_output():
+            with suppress_output():
                 split(
                     vis=msname,
                     field=str(field),
@@ -196,7 +183,7 @@ def do_selfcal(
         ##########################################
         # Initiate proper weighting
         ##########################################
-        with suppress_casa_output():
+        with suppress_output():
             flagdata(
                 vis=msname,
                 mode="clip",
@@ -564,9 +551,10 @@ def main(
     keep_backup=False,
     cpu_frac=0.8,
     mem_frac=0.8,
+    logfile=None,
     jobid=0,
     start_remote_log=False,
-    dask_addr=None,
+    dask_client=None,
 ):
     """
     Perform iterative self-calibration on a list of measurement sets.
@@ -615,12 +603,14 @@ def main(
         Fraction of available CPUs to use per job. Default is 0.8.
     mem_frac : float, optional
         Fraction of available system memory to use per job. Default is 0.8.
+    logfile : str, optional
+        Log file name
     jobid : int, optional
         Identifier for job tracking and logging. Default is 0.
     start_remote_log : bool, optional
         Whether to initiate remote logging via job credentials. Default is False.
-    dask_addr : str, optional
-        Dask scheduler address
+    dask_client : dask.client, optional
+        Dask client
 
     Returns
     -------
@@ -640,11 +630,6 @@ def main(
         caldir = f"{workdir}/caltables"
     os.makedirs(caldir, exist_ok=True)
 
-    os.makedirs(workdir + "/logs", exist_ok=True)
-    mainlog_file = workdir + "/logs/selfcal_targets.mainlog"
-    mainlogger, mainlog_file = create_logger(
-        os.path.basename(mainlog_file).split(".mainlog")[0], mainlog_file, verbose=False
-    )
     ############
     # Logger
     ############
@@ -652,19 +637,29 @@ def main(
     if (
         start_remote_log
         and os.path.exists(f"{workdir}/jobname_password.npy")
-        and mainlog_file is not None
+        and logfile is not None
     ):
         time.sleep(5)
         jobname, password = np.load(
             f"{workdir}/jobname_password.npy", allow_pickle=True
         )
-        if os.path.exists(mainlog_file):
+        if os.path.exists(logfile):
             observer = init_logger(
-                "all_selfcal", mainlog_file, jobname=jobname, password=password
+                "all_selfcal", logfile, jobname=jobname, password=password
             )
     if observer == None:
         print("Remote link or jobname is blank. Not transmiting to remote logger.")
-    ###########
+
+    dask_cluster = None
+    if dask_client is None:
+        dask_client, dask_cluster, dask_dir = get_local_dask_cluster(
+            2,
+            dask_dir=workdir,
+            cpu_frac=cpu_frac,
+            mem_frac=mem_frac,
+        )
+        nworker = max(2, int(psutil.cpu_count() * cpu_frac))
+        scale_worker_and_wait(dask_cluster, nworker)
 
     ###########################
     # WSClean container
@@ -681,10 +676,9 @@ def main(
     org_mslist = copy.deepcopy(mslist)
     try:
         if len(mslist) == 0:
-            mainlogger.info("Please provide at-least one measurement set.")
+            print("Please provide at-least one measurement set.")
             msg = 1
         else:
-            mem_limit = do_selfcal(dry_run=True)
             partial_do_selfcal = partial(
                 do_selfcal,
                 start_threshold=float(start_thresh),
@@ -713,7 +707,7 @@ def main(
                 if checkcol:
                     filtered_mslist.append(ms)
                 else:
-                    mainlogger.info(f"Issue in : {ms}")
+                    print(f"Issue in : {ms}")
                     os.system("rm -rf {ms}")
             mslist = filtered_mslist
 
@@ -728,12 +722,6 @@ def main(
                 if channame not in chanlist:
                     chanlist.append(channame)
 
-            available_mem = psutil.virtual_memory().available / 1024**3
-            if mem_limit < 4 and available_mem > 4:
-                min_mem_per_job = 4
-            else:
-                min_mem_per_job = mem_limit
-
             ######################################
             # Resetting maximum file limit
             ######################################
@@ -744,7 +732,7 @@ def main(
 
             num_fd_list = []
             if len(mslist) == 0:
-                mainlogger.error("No filtered ms to continue.")
+                print("No filtered ms to continue.")
                 return 1
             else:
                 for ms in mslist:
@@ -766,34 +754,30 @@ def main(
                         per_job_fd = 1
                     num_fd_list.append(per_job_fd)
                 total_fd = max(num_fd_list) * len(mslist)
-                n_jobs = max(1, int(new_soft_limit / total_fd))
-                n_jobs = min(len(mslist), n_jobs)
-                if dask_addr is None:
-                    dask_client, dask_cluster, n_jobs, n_threads, mem_limit, dask_dir = (
-                        get_dask_client(
-                            n_jobs,
-                            dask_dir=workdir,
-                            cpu_frac=float(cpu_frac),
-                            mem_frac=float(mem_frac),
-                            min_cpu_per_job=3,
-                            min_mem_per_job=min_mem_per_job,
-                        )
-                    )
-                else:
-                    _, _, n_jobs, n_threads, mem_limit, dask_dir = (
-                        get_dask_client(
-                            n_jobs,
-                            dask_dir=workdir,
-                            cpu_frac=float(cpu_frac),
-                            mem_frac=float(mem_frac),
-                            min_cpu_per_job=3,
-                            min_mem_per_job=min_mem_per_job,
-                            only_cal=True,
-                        )
-                    )
-                    dask_client=Client(address=dask_addr)
-                    os.system(f"rm -rf {dask_dir}")
-                wait_for_dask_workers(dask_client,min_worker=1,timeout=60)
+
+                if cpu_frac > 0.8:
+                    cpu_frac = 0.8
+                total_cpu = max(1, int(psutil.cpu_count() * cpu_frac))
+                if mem_frac > 0.8:
+                    mem_frac = 0.8
+                total_mem = (psutil.virtual_memory().available * mem_frac) / (
+                    1024**3
+                )  # In GB
+                njobs = min(len(mslist), int(new_soft_limit / total_fd))
+                njobs = max(1, min(total_cpu, njobs))
+
+                #####################################
+                # Determining per jobs resource
+                #####################################
+                n_threads = max(1, int(total_cpu / njobs))
+                mem_limit = total_mem / njobs
+                print("#################################")
+                print(f"Total dask worker: {njobs}")
+                print(f"CPU per worker: {n_threads}")
+                print(f"Memory per worker: {round(mem_limit,2)} GB")
+                print("#################################")
+
+                #####################################
                 tasks = []
                 for ms in mslist:
                     logfile = (
@@ -802,7 +786,7 @@ def main(
                         + os.path.basename(ms).split(".ms")[0]
                         + "_selfcal.log"
                     )
-                    mainlogger.info(f"MS name: {ms}, Log file: {logfile}\n")
+                    print(f"MS name: {ms}, Log file: {logfile}")
                     tasks.append(
                         delayed(partial_do_selfcal)(
                             ms,
@@ -816,18 +800,15 @@ def main(
                             logfile=logfile,
                         )
                     )
-                futures = dask_client.compute(tasks)
-                results = list(dask_client.gather(futures))
-                dask_client.close()
-                if dask_addr is None:
-                    dask_cluster.close()
-                    os.system(f"rm -rf {dask_dir}")
+                print("Starting all self-calibration...")
+                results = list(dask_client.gather(dask_client.compute(tasks)))
+
                 gcal_list = []
                 for i in range(len(results)):
                     r = results[i]
                     msg = r[0]
                     if msg != 0:
-                        mainlogger.info(
+                        print(
                             f"Self-calibration was not successful for ms: {mslist[i]}."
                         )
                     else:
@@ -849,10 +830,10 @@ def main(
                         )
                         os.system("rm -rf " + selfcaldir)
                 if len(gcal_list) > 0:
-                    mainlogger.info(f"Final selfcal caltables: {gcal_list}")
+                    print(f"Final selfcal caltables: {gcal_list}")
                     msg = 0
                 else:
-                    mainlogger.info("No self-calibration is successful.")
+                    print("No self-calibration is successful.")
                     msg = 1
     except Exception as e:
         traceback.print_exc()
@@ -863,6 +844,10 @@ def main(
             drop_cache(ms)
         drop_cache(workdir)
         clean_shutdown(observer)
+        if dask_cluster is not None:
+            dask_client.close()
+            dask_cluster.close()
+            os.system(f"rm -rf {dask_dir}")
     return msg
 
 
@@ -1020,7 +1005,7 @@ def cli():
 
     if len(sys.argv) == 1:
         parser.print_help(sys.stderr)
-        sys.exit(1)
+        return 1
 
     args = parser.parse_args()
 
