@@ -7,21 +7,12 @@ import traceback
 import time
 import sys
 import os
-from casatasks import casalog
 from casatools import msmetadata
 from dask import delayed
-from dask.distributed import Client
 from meersolar.utils import *
 
-logging.getLogger("distributed").setLevel(logging.WARNING)
-
-
-try:
-    casalogfile = casalog.logfile()
-    os.system("rm -rf " + casalogfile)
-except BaseException:
-    pass
-
+logging.getLogger("distributed").setLevel(logging.ERROR)
+logging.getLogger("tornado.application").setLevel(logging.CRITICAL)
 datadir = get_datadir()
 
 
@@ -45,6 +36,7 @@ def chanlist_to_str(lst):
 
 def split_target_scans(
     msname,
+    dask_client,
     workdir,
     timeres,
     freqres,
@@ -60,9 +52,6 @@ def split_target_scans(
     merge_spws=False,
     cpu_frac=0.8,
     mem_frac=0.8,
-    max_cpu_frac=0.8,
-    max_mem_frac=0.8,
-    dask_addr=None,
 ):
     """
     Split target scans
@@ -71,6 +60,8 @@ def split_target_scans(
     ----------
     msname : str
         Measurement set
+    dask_client : dask.client
+        Dask client
     workdir : str
         Work directory
     timeres : float
@@ -101,20 +92,20 @@ def split_target_scans(
         CPU fraction to use
     mem_frac : float, optional
         Memory fraction to use
-    max_cpu_frac : float, optional
-        Maximum CPU fraction to use
-    max_mem_frac : float, optional
-        Maximum memory fraction to use
-    dask_addr : str, optional
-        Dask scheduler address
 
     Returns
     -------
     list
         Splited ms list
     """
-    start_time = time.time()
     try:
+        if cpu_frac > 0.8:
+            cpu_frac = 0.8
+        total_cpu = max(1, int(psutil.cpu_count() * cpu_frac))
+        if mem_frac > 0.8:
+            mem_frac = 0.8
+        total_mem = (psutil.virtual_memory().available * mem_frac) / (1024**3)  # In GB
+
         os.chdir(workdir)
         print(f"Spliting ms : {msname}")
         target_scans, cal_scans, f_scans, g_scans, p_scans = get_cal_target_scans(
@@ -229,22 +220,9 @@ def split_target_scans(
         else:
             total_chunks = len(filtered_scan_list)
 
-        #############################################
-        # Memory limit
-        #############################################
-        mem_limit = single_mstransform(dry_run=True)
-        #######################
-        _, _, max_n_jobs, n_threads, mem_limit, dask_dir = (
-            get_dask_client(
-                total_chunks,
-                dask_dir=workdir,
-                cpu_frac=max_cpu_frac,
-                mem_frac=max_mem_frac,
-                min_mem_per_job=mem_limit,
-                only_cal=True,
-            )
-        )
-        os.system(f"rm -rf {dask_dir}")
+        njobs = max(1, min(total_cpu, total_chunks))
+        n_threads = max(1, int(total_cpu / njobs))
+
         tasks = []
         splited_ms_list = []
         for scan in filtered_scan_list:
@@ -279,129 +257,27 @@ def split_target_scans(
                         corr="",
                         timerange=timerange,
                         n_threads=n_threads,
-                        dry_run=False,
                     )
                     tasks.append(task)
-        #####################################
-        # Adaptive dask client
-        #####################################
-        if cpu_frac == max_cpu_frac and mem_frac == max_mem_frac:
-            total_chunks = len(tasks)
-            if total_chunks > 0:
-                if dask_addr is None:
-                    dask_client, dask_cluster, n_jobs, n_threads, mem_limit, dask_dir = (
-                        get_dask_client(
-                            total_chunks,
-                            dask_dir=workdir,
-                            cpu_frac=cpu_frac,
-                            mem_frac=mem_frac,
-                            min_mem_per_job=mem_limit,
-                        )
-                    )
-                else:
-                    _, _, n_jobs, n_threads, mem_limit, dask_dir = (
-                        get_dask_client(
-                            total_chunks,
-                            dask_dir=workdir,
-                            cpu_frac=cpu_frac,
-                            mem_frac=mem_frac,
-                            min_mem_per_job=mem_limit,
-                            only_cal=True,
-                        )
-                    )
-                    dask_client=Client(address=dask_addr)
-                    os.system(f"rm -rf {dask_dir}")
-                wait_for_dask_workers(dask_client,min_worker=1,timeout=60)
-                futures = dask_client.compute(tasks)
-                results = list(dask_client.gather(futures))
-                dask_client.close()
-                if dask_addr is None:
-                    dask_cluster.close()
-                    os.system(f"rm -rf {dask_dir}")
-                for r in results:
-                    splited_ms_list.append(r)
-        else:
-            while True:
-                total_chunks = len(tasks)
-                if total_chunks == 0:
-                    break
-                else:
-                    if dask_addr is None:
-                        (
-                            dask_client,
-                            dask_cluster,
-                            n_jobs,
-                            n_threads,
-                            mem_limit,
-                            dask_dir,
-                        ) = get_dask_client(
-                            total_chunks,
-                            dask_dir=workdir,
-                            cpu_frac=cpu_frac,
-                            mem_frac=mem_frac,
-                            min_mem_per_job=mem_limit,
-                        )
-                    else:
-                        (
-                            _,
-                            _,
-                            n_jobs,
-                            n_threads,
-                            mem_limit,
-                            dask_dir,
-                        ) = get_dask_client(
-                            total_chunks,
-                            dask_dir=workdir,
-                            cpu_frac=cpu_frac,
-                            mem_frac=mem_frac,
-                            min_mem_per_job=mem_limit,
-                            only_cal=True,
-                        )
-                        dask_client=Client(address=dask_addr)
-                        os.system(f"rm -rf {dask_dir}")
-                    wait_for_dask_workers(dask_client,min_worker=1,timeout=60)
-                    chunk_tasks = tasks[0 : min(n_jobs, max_n_jobs)]
-                    for ctask in chunk_tasks:
-                        tasks.remove(ctask)
-                    futures = dask_client.compute(tasks)
-                    results = list(dask_client.gather(futures))
-                    dask_client.close()
-                    if dask_addr is None:
-                        dask_cluster.close()
-                        os.system(f"rm -rf {dask_dir}")
-                    for r in results:
-                        splited_ms_list.append(r)
-                    n_current_process = (
-                        get_nprocess_solarpipe(workdir) - 1
-                    )  # One is subtracted for the current process
-                    if len(tasks) == 0:
-                        break
-                    elif n_current_process == 0:
-                        available_cpu_frac = round(
-                            (100 - psutil.cpu_percent(interval=1)) / 100.0, 2
-                        )
-                        available_mem_frac = round(
-                            psutil.virtual_memory().available
-                            / psutil.virtual_memory().total,
-                            2,
-                        )
-                        cpu_frac = min(max_cpu_frac, max(cpu_frac, available_cpu_frac))
-                        mem_frac = min(max_mem_frac, max(mem_frac, available_mem_frac))
-                        print(
-                            f"Updated CPU fraction: {cpu_frac}, memory fraction: {mem_frac}."
-                        )
+        if len(tasks):
+            print("Start spliting..")
+            futures = dask_client.compute(tasks)
+            results = list(dask_client.gather(futures))
+            for splited_ms in results:
+                splited_ms_list.append(splited_ms)
         print("##################")
         print("Spliting of target scans are done successfully.")
-        print("Total time taken : ", time.time() - start_time)
-        print("##################\n")
+        print("##################")
         return 0, splited_ms_list
     except Exception as e:
         traceback.print_exc()
         print("##################")
         print("Spliting of target scans are unsuccessful.")
-        print("Total time taken : ", time.time() - start_time)
-        print("##################\n")
+        print("##################")
         return 1, []
+    finally:
+        time.sleep(1)
+        drop_cache(msname)
 
 
 def main(
@@ -421,12 +297,10 @@ def main(
     merge_spws=False,
     cpu_frac=0.8,
     mem_frac=0.8,
-    max_cpu_frac=0.8,
-    max_mem_frac=0.8,
     logfile=None,
     jobid=0,
     start_remote_log=False,
-    dask_addr=None,
+    dask_client=None,
 ):
     """
     Split target scans from a measurement set into smaller chunks for parallel processing.
@@ -465,18 +339,14 @@ def main(
         Fraction of available CPUs to allocate per task. Default is 0.8.
     mem_frac : float, optional
         Fraction of available memory to allocate per task. Default is 0.8.
-    max_cpu_frac : float, optional
-        Maximum total CPU usage across all tasks. Default is 0.8.
-    max_mem_frac : float, optional
-        Maximum total memory usage across all tasks. Default is 0.8.
     logfile : str or None, optional
         Path to log file. If None, logging to file is disabled. Default is None.
     jobid : int, optional
         Job identifier for tracking and PID storage. Default is 0.
     start_remote_log : bool, optional
         If True, enables remote logging using credentials stored in workdir. Default is False.
-    dask_addr : str, optional
-        Dask scheduler address
+    dask_client : dask.client, optional
+        Dask client
 
     Returns
     -------
@@ -510,16 +380,27 @@ def main(
             )
     if observer == None:
         print("Remote link or jobname is blank. Not transmiting to remote logger.")
-    ###########
+
+    dask_cluster = None
+    if dask_client is None:
+        dask_client, dask_cluster, dask_dir = get_local_dask_cluster(
+            2,
+            dask_dir=workdir,
+            cpu_frac=cpu_frac,
+            mem_frac=mem_frac,
+        )
+        nworker = max(2, int(psutil.cpu_count() * cpu_frac))
+        scale_worker_and_wait(dask_cluster, nworker)
 
     try:
         if msname and os.path.exists(msname):
-            print("\n###################################")
+            print("###################################")
             print("Start spliting target scans.")
-            print("###################################\n")
+            print("###################################")
             scans = [int(i) for i in scans.split(",")] if scans else []
             msg, final_target_mslist = split_target_scans(
                 msname,
+                dask_client,
                 workdir,
                 float(timeres),
                 float(freqres),
@@ -535,12 +416,9 @@ def main(
                 spectral_chunk=float(spectral_chunk),
                 cpu_frac=float(cpu_frac),
                 mem_frac=float(mem_frac),
-                max_cpu_frac=float(max_cpu_frac),
-                max_mem_frac=float(max_mem_frac),
-                dask_addr=dask_addr,
             )
         else:
-            print("Please provide correct measurement set.\n")
+            print("Please provide correct measurement set.")
             msg = 1
     except Exception as e:
         traceback.print_exc()
@@ -550,6 +428,10 @@ def main(
         drop_cache(msname)
         drop_cache(workdir)
         clean_shutdown(observer)
+        if dask_cluster is not None:
+            dask_client.close()
+            dask_cluster.close()
+            os.system(f"rm -rf {dask_dir}")
     return msg
 
 
@@ -674,26 +556,12 @@ def cli():
         help="Memory fraction to use",
         metavar="Float",
     )
-    hard_args.add_argument(
-        "--max_cpu_frac",
-        type=float,
-        default=0.8,
-        help="Maximum CPU fraction to use",
-        metavar="Float",
-    )
-    hard_args.add_argument(
-        "--max_mem_frac",
-        type=float,
-        default=0.8,
-        help="Maximum memory fraction to use",
-        metavar="Float",
-    )
     hard_args.add_argument("--logfile", type=str, default=None, help="Log file")
     hard_args.add_argument("--jobid", type=int, default=0, help="Job ID")
 
     if len(sys.argv) == 1:
         parser.print_help(sys.stderr)
-        sys.exit(1)
+        return 1
 
     args = parser.parse_args()
 
@@ -714,8 +582,6 @@ def cli():
         merge_spws=args.merge_spws,
         cpu_frac=args.cpu_frac,
         mem_frac=args.mem_frac,
-        max_cpu_frac=args.max_cpu_frac,
-        max_mem_frac=args.max_mem_frac,
         logfile=args.logfile,
         jobid=args.jobid,
         start_remote_log=args.start_remote_log,
