@@ -8,20 +8,12 @@ import time
 import glob
 import sys
 import os
-from casatasks import casalog
 from casatools import msmetadata
 from dask import delayed
-from dask.distributed import Client
 from meersolar.utils import *
 
-logging.getLogger("distributed").setLevel(logging.WARNING)
-
-try:
-    casalogfile = casalog.logfile()
-    os.system("rm -rf " + casalogfile)
-except BaseException:
-    pass
-
+logging.getLogger("distributed").setLevel(logging.ERROR)
+logging.getLogger("tornado.application").setLevel(logging.CRITICAL)
 datadir = get_datadir()
 
 
@@ -36,7 +28,6 @@ def single_ms_flag(
     flag_autocorr=True,
     n_threads=-1,
     memory_limit=-1,
-    dry_run=False,
 ):
     """
     Flag on a single ms
@@ -63,25 +54,18 @@ def single_ms_flag(
         Number of OpenMP threads
     memory_limit : float, optional
         Memory limit in GB
-    dry_run : bool, optional
-        Return the amount of pre-occupied memory in GB
     """
     limit_threads(n_threads=n_threads)
     from casatasks import flagdata
 
-    if dry_run:
-        process = psutil.Process(os.getpid())
-        mem = round(process.memory_info().rss / 1024**3, 2)  # in GB
-        return mem
     msname = msname.rstrip("/")
     try:
         ##############################
         # Flagging bad channels
         ##############################
         if badspw != "":
-            print(f"Flagging bad spectral windows: {badspw}\n")
             try:
-                with suppress_casa_output():
+                with suppress_output():
                     flagdata(
                         vis=msname,
                         mode="manual",
@@ -96,9 +80,8 @@ def single_ms_flag(
         # Flagging bad antennas
         ##############################
         if bad_ants_str != "":
-            print(f"Flagging bad antenna: {bad_ants_str}\n")
             try:
-                with suppress_casa_output():
+                with suppress_output():
                     flagdata(
                         vis=msname,
                         mode="manual",
@@ -113,7 +96,7 @@ def single_ms_flag(
         # Clip zero amplitude data points
         #################################
         try:
-            with suppress_casa_output():
+            with suppress_output():
                 flagdata(
                     vis=msname,
                     mode="clip",
@@ -130,7 +113,7 @@ def single_ms_flag(
         #################################
         if flag_autocorr:
             try:
-                with suppress_casa_output():
+                with suppress_output():
                     flagdata(
                         vis=msname,
                         mode="manual",
@@ -170,7 +153,7 @@ def single_ms_flag(
             )
             if not corcolumn_present:
                 print(
-                    "Corrected data column is chosen for flagging, but it is not present.\n"
+                    "Corrected data column is chosen for flagging, but it is not present."
                 )
                 return
             else:
@@ -182,7 +165,7 @@ def single_ms_flag(
         if datacolumn == "data" or datacolumn == "DATA":
             datacolumn_present = check_datacolumn_valid(msname, datacolumn="DATA")
             if not datacolumn_present:
-                print("Data column is chosen for flagging, but it is not present.\n")
+                print("Data column is chosen for flagging, but it is not present.")
                 return
             else:
                 datacolumn = "data"
@@ -211,7 +194,7 @@ def single_ms_flag(
         ##############
         if use_tfcrop:
             try:
-                with suppress_casa_output():
+                with suppress_output():
                     flagdata(
                         vis=msname,
                         mode="tfcrop",
@@ -237,7 +220,7 @@ def single_ms_flag(
         # Rflag flag
         #############
         try:
-            with suppress_casa_output():
+            with suppress_output():
                 flagdata(
                     vis=msname,
                     mode="rflag",
@@ -262,7 +245,7 @@ def single_ms_flag(
         # Extend flag
         ##############
         try:
-            with suppress_casa_output():
+            with suppress_output():
                 flagdata(
                     vis=msname,
                     mode="extend",
@@ -291,6 +274,7 @@ def single_ms_flag(
 
 def do_flagging(
     msname,
+    dask_client,
     workdir,
     datacolumn="data",
     flag_bad_ants=True,
@@ -302,7 +286,6 @@ def do_flagging(
     flag_backup=True,
     cpu_frac=0.8,
     mem_frac=0.8,
-    dask_addr=None,
 ):
     """
     Function to perform initial flagging
@@ -311,6 +294,8 @@ def do_flagging(
     ----------
     msname : str
         Name of the ms
+    dask_client : dask.client
+        Dask client
     workdir : str
         Work directory
     datacolumn : str, optional
@@ -333,16 +318,20 @@ def do_flagging(
         CPU fraction to use
     mem_frac : float, optional
         Memory fraction to use
-    dask_addr : str, optional
-        Dask scheduler address
 
     Returns
     -------
     int
         Success message
     """
-    start_time = time.time()
     try:
+        if cpu_frac > 0.8:
+            cpu_frac = 0.8
+        total_cpu = max(1, int(psutil.cpu_count() * cpu_frac))
+        if mem_frac > 0.8:
+            mem_frac = 0.8
+        total_mem = (psutil.virtual_memory().available * mem_frac) / (1024**3)  # In GB
+
         from casatasks import flagdata
 
         msname = msname.rstrip("/")
@@ -350,12 +339,10 @@ def do_flagging(
         os.chdir(mspath)
         print("###########################")
         print("Flagging measurement set : ", msname)
-        print("###########################\n")
-        total_cpus = psutil.cpu_count(logical=True)
-        ncpu = int(total_cpus * cpu_frac)
+        print("###########################")
         correct_missing_col_subms(msname)
         print("Restoring all previous flags...")
-        with suppress_casa_output():
+        with suppress_output():
             flagdata(vis=msname, mode="unflag", spw="0", flagbackup=False)
         fluxcal_field, fluxcal_scans = get_fluxcals(msname)
         if len(fluxcal_field) == 0:
@@ -369,39 +356,24 @@ def do_flagging(
             bad_ants, bad_ants_str = get_bad_ants(msname, fieldnames=fluxcal_field)
         else:
             bad_ants_str = ""
-        ###########################
-        # Dask local cluster setup
-        ##########################
         if os.path.exists(msname + "/SUBMSS"):
             subms_list = glob.glob(msname + "/SUBMSS/*")
             for subms in subms_list:
                 os.system(f"rm -rf {subms}/.flagversions")
         else:
             subms_list = [msname]
-        ms_size_list = [get_column_size(ms) for ms in subms_list]
-        mem_limit = max(ms_size_list)
-        if dask_addr is None:
-            dask_client, dask_cluster, n_jobs, n_threads, mem_limit, dask_dir = (
-                get_dask_client(
-                    len(subms_list),
-                    dask_dir=workdir,
-                    cpu_frac=cpu_frac,
-                    mem_frac=mem_frac,
-                    min_mem_per_job=mem_limit,
-                )
-            )
-        else:
-            _, _, n_jobs, n_threads, mem_limit, dask_dir = get_dask_client(
-                len(subms_list),
-                dask_dir=workdir,
-                cpu_frac=cpu_frac,
-                mem_frac=mem_frac,
-                min_mem_per_job=mem_limit,
-                only_cal=True,
-            )
-            os.system(f"rm -rf {dask_dir}")
-            dask_client = Client(address=dask_addr)
-        wait_for_dask_workers(dask_client,min_worker=1,timeout=60)
+
+        njobs = max(1, min(total_cpu, len(subms_list)))
+        n_threads = max(1, int(total_cpu / njobs))
+        mem_limit = total_mem / njobs
+
+        print("#################################")
+        print(f"Total dask worker: {njobs}")
+        print(f"CPU per worker: {n_threads}")
+        print(f"Memory per worker: {round(mem_limit,2)} GB")
+        print("#################################")
+        ###########################################
+
         if flag_backup:
             do_flag_backup(msname, flagtype="flagdata")
         tasks = [
@@ -419,21 +391,12 @@ def do_flagging(
             )
             for ms in subms_list
         ]
+        print(f"Flagging mslist: {','.join(subms_list)}")
         futures = dask_client.compute(tasks)
         results = list(dask_client.gather(futures))
-        dask_client.close()
-        if dask_addr is None:
-            dask_cluster.close()
-            os.system(f"rm -rf {dask_dir}")
-        print("##################")
-        print("Total time taken : ", time.time() - start_time)
-        print("##################\n")
         return 0
     except Exception as e:
         traceback.print_exc()
-        print("##################")
-        print("Total time taken : " + str(time.time() - start_time) + "s")
-        print("##################\n")
         return 1
 
 
@@ -453,7 +416,7 @@ def main(
     logfile=None,
     jobid=0,
     start_remote_log=False,
-    dask_addr=None,
+    dask_client=None,
 ):
     """
     Run the flagging pipeline for a measurement set.
@@ -491,8 +454,8 @@ def main(
         Numeric job ID used for PID tracking. Default is 0.
     start_remote_log : bool, optional
         Whether to enable remote logging using credentials in the workdir. Default is False.
-    dask_addr : str, optional
-        Dask scheduler address
+    dask_client : dask.client, optional
+        Dask client
 
     Returns
     -------
@@ -516,7 +479,7 @@ def main(
         and os.path.exists(f"{workdir}/jobname_password.npy")
         and logfile is not None
     ):
-        time.sleep(5)
+        time.sleep(1)
         jobname, password = np.load(
             f"{workdir}/jobname_password.npy", allow_pickle=True
         )
@@ -526,12 +489,23 @@ def main(
             )
     if observer == None:
         print("Remote link or jobname is blank. Not transmiting to remote logger.")
-    ###########
+
+    dask_cluster = None
+    if dask_client is None:
+        dask_client, dask_cluster, dask_dir = get_local_dask_cluster(
+            2,
+            dask_dir=workdir,
+            cpu_frac=cpu_frac,
+            mem_frac=mem_frac,
+        )
+        nworker = max(2, int(psutil.cpu_count() * cpu_frac))
+        scale_worker_and_wait(dask_cluster, nworker)
 
     try:
         if msname and os.path.exists(msname):
             msg = do_flagging(
                 msname,
+                dask_client,
                 workdir,
                 datacolumn=datacolumn,
                 flag_bad_ants=flag_bad_ants,
@@ -543,19 +517,22 @@ def main(
                 flag_backup=flagbackup,
                 cpu_frac=cpu_frac,
                 mem_frac=mem_frac,
-                dask_addr=dask_addr,
             )
         else:
-            print("Please provide correct measurement set.\n")
+            print("Please provide correct measurement set.")
             msg = 1
     except Exception as e:
         traceback.print_exc()
         msg = 1
     finally:
-        time.sleep(5)
+        time.sleep(1)
         drop_cache(msname)
         drop_cache(workdir)
         clean_shutdown(observer)
+        if dask_cluster is not None:
+            dask_client.close()
+            dask_cluster.close()
+            os.system(f"rm -rf {dask_dir}")
     return msg
 
 
@@ -631,7 +608,7 @@ def cli():
 
     if len(sys.argv) == 1:
         parser.print_help(sys.stderr)
-        sys.exit(1)
+        return 1
 
     args = parser.parse_args()
 
@@ -657,6 +634,5 @@ def cli():
 
 if __name__ == "__main__":
     result = cli()
-    print(f"Final msg : {result}")
-    print("\n###################\nBasic flagging is finished.\n###################\n")
+    print("\n###################\nFlagging is done.\n###################\n")
     os._exit(result)
