@@ -7,21 +7,11 @@ import time
 import sys
 import os
 import logging
-from casatasks import casalog
 from dask import delayed
-from dask.distributed import Client
 from meersolar.utils import *
 
-logging.getLogger("distributed").setLevel(logging.WARNING)
-
-
-try:
-    casalogfile = casalog.logfile()
-    os.system("rm -rf " + casalogfile)
-except BaseException:
-    pass
-
-
+logging.getLogger("distributed").setLevel(logging.ERROR)
+logging.getLogger("tornado.application").setLevel(logging.CRITICAL)
 datadir = get_datadir()
 
 
@@ -56,7 +46,7 @@ def get_polmodel_coeff(model_file):
     return ref_freq, I[0], polyI[1:], poly_pfrac, poly_pangle
 
 
-def polcal_setjy(msname="", field_name="", n_threads=-1, ismms=True, dry_run=False):
+def polcal_setjy(msname="", field_name="", n_threads=-1, ismms=True):
     """
     Setjy polcal fields (3C286 or 3C138)
 
@@ -72,10 +62,6 @@ def polcal_setjy(msname="", field_name="", n_threads=-1, ismms=True, dry_run=Fal
     limit_threads(n_threads=n_threads)
     from casatasks import setjy
 
-    if dry_run:
-        process = psutil.Process(os.getpid())
-        mem = round(process.memory_info().rss / 1024**3, 2)  # in GB
-        return mem
     try:
         print(f"Polarization calibrator field name : {field_name}")
         if field_name in ["3C286", "1328+307", "1331+305", "J1331+3030"]:
@@ -90,7 +76,7 @@ def polcal_setjy(msname="", field_name="", n_threads=-1, ismms=True, dry_run=Fal
             print("Field name is not either of the polcal, 3C286 or 3C138.")
             return 1
         print(f"Reference frequency: {ref_freq} GHz.")
-        with suppress_casa_output():
+        with suppress_output():
             setjy(
                 vis=msname,
                 field=field_name,
@@ -109,7 +95,7 @@ def polcal_setjy(msname="", field_name="", n_threads=-1, ismms=True, dry_run=Fal
     return
 
 
-def phasecal_setjy(msname="", field="", ismms=False, n_threads=-1, dry_run=False):
+def phasecal_setjy(msname="", field="", ismms=False, n_threads=-1):
     """
     Setjy phasecal fields
 
@@ -127,12 +113,8 @@ def phasecal_setjy(msname="", field="", ismms=False, n_threads=-1, dry_run=False
     limit_threads(n_threads=n_threads)
     from casatasks import setjy
 
-    if dry_run:
-        process = psutil.Process(os.getpid())
-        mem = round(process.memory_info().rss / 1024**3, 2)  # in GB
-        return mem
     try:
-        with suppress_casa_output():
+        with suppress_output():
             setjy(
                 vis=msname,
                 field=field,
@@ -170,7 +152,7 @@ def import_fluxcal_models(mslist, fluxcal_fields, fluxcal_scans, ncpus=1, mem_fr
     """
     try:
         if len(fluxcal_fields) == 0:
-            print("No flux calibrator scan is present.\n")
+            print("No flux calibrator scan is present.")
             return 1
         print("##############################")
         print("Import fluxcal models")
@@ -198,15 +180,16 @@ def import_fluxcal_models(mslist, fluxcal_fields, fluxcal_scans, ncpus=1, mem_fr
                         "-ns 1000",
                         "-j " + str(ncpus),
                         "-mf " + str(mem_frac),
+                        sub_msname,
                     ]
-                    crys_cmd = (
-                        "crystalball " + " ".join(crys_cmd_args) + " " + sub_msname
-                    )
-                    print(crys_cmd)
-                    tmpfile = f"tmp_{os.path.basename(sub_msname).split('.ms')[0]}"
-                    with suppress_casa_output():
-                        msg = os.system(crys_cmd + f" > {tmpfile}")
-                    os.system(f"rm -rf {tmpfile}")
+                    cmd = ["crystalball"] + crys_cmd_args
+                    cmd = " ".join(cmd)
+                    print(f"Running: {cmd}")
+                    tmpfile = f"tmp_{os.path.basename(sub_msname)}"
+                    with suppress_output():
+                        msg = os.system(f"{cmd} > {tmpfile}")
+                    if os.path.exists(tmpfile):
+                        os.remove(tmpfile)
                     if msg == 0:
                         print(f"Fluxcal model is imported successfully for scan: {s}.")
                     else:
@@ -220,12 +203,11 @@ def import_fluxcal_models(mslist, fluxcal_fields, fluxcal_scans, ncpus=1, mem_fr
 
 def import_phasecal_models(
     mslist,
+    dask_client,
     phasecal_fields,
     phasecal_scans,
     workdir,
     cpu_frac=0.8,
-    mem_frac=0.8,
-    dask_addr=None,
 ):
     """
     Import model visibilities for phasecal
@@ -234,6 +216,8 @@ def import_phasecal_models(
     ----------
     mslist : list
         List of sub-MS
+    dask_client : dask.client
+        Dask client
     phasecal_fields : list
         Phasecal fields
     phasecal_scans : list
@@ -242,10 +226,6 @@ def import_phasecal_models(
         Work directory
     cpu_frac : float, optional
         CPU fraction to use
-    mem_frac : float, optional
-        Memory fraction to use
-    dask_addr : str, optional
-        Dask scheduler address
 
     Returns
     -------
@@ -253,6 +233,9 @@ def import_phasecal_models(
         Success message
     """
     try:
+        if cpu_frac > 0.8:
+            cpu_frac = 0.8
+        total_cpu = max(1, int(psutil.cpu_count() * cpu_frac))
         print("##########################")
         print("Import phasecal models")
         print("##########################")
@@ -261,30 +244,14 @@ def import_phasecal_models(
             ms_scans = get_ms_scans(ms)
             for s in ms_scans:
                 scans.append(s)
-        mem_limit = phasecal_setjy(dry_run=True)
-        if dask_addr is None:
-            dask_client, dask_cluster, n_jobs, n_threads, mem_limit, dask_dir = (
-                get_dask_client(
-                    len(mslist),
-                    dask_dir=workdir,
-                    cpu_frac=cpu_frac,
-                    mem_frac=mem_frac,
-                    min_mem_per_job=mem_limit,
-                )
-            )
-        else:
-            _, _, n_jobs, n_threads, mem_limit, dask_dir = get_dask_client(
-                len(mslist),
-                dask_dir=workdir,
-                cpu_frac=cpu_frac,
-                mem_frac=mem_frac,
-                min_mem_per_job=mem_limit,
-                only_cal=True,
-            )
-            os.system(f"rm -rf {dask_dir}")
-            dask_client = Client(address=dask_addr)
-        wait_for_dask_workers(dask_client,min_worker=1,timeout=60)
         tasks = []
+        total_scans = 0
+        for phasecal in phasecal_fields:
+            ph_scan = phasecal_scans[phasecal]
+            for s in ph_scan:
+                if s in scans:
+                    total_scans += 1
+        n_threads = max(1, int(total_cpu / total_scans))
         for phasecal in phasecal_fields:
             ph_scan = phasecal_scans[phasecal]
             for s in ph_scan:
@@ -299,13 +266,9 @@ def import_phasecal_models(
                             n_threads=n_threads,
                         )
                     )
-        futures = dask_client.compute(tasks)
-        results = list(dask_client.gather(futures))
-        dask_client.close()
-        if dask_addr is None:
-            dask_cluster.close()
-            os.system(f"rm -rf {dask_dir}")
-        print("Phasecal models are initiated.\n")
+        print("Start importing phasecal models...")
+        results = list(dask_client.gather(dask_client.compute(tasks)))
+        print("Phasecal models are initiated.")
         return 0
     except Exception as e:
         print("Error in initiaing phasecal models.")
@@ -315,12 +278,11 @@ def import_phasecal_models(
 
 def import_polcal_models(
     mslist,
+    dask_client,
     polcal_fields,
     polcal_scans,
     workdir,
     cpu_frac=0.8,
-    mem_frac=0.8,
-    dask_addr=None,
 ):
     """
     Import model for polarization calibrators (3C286 or 3C138)
@@ -329,16 +291,16 @@ def import_polcal_models(
     ----------
     mslist : list
         List of sub-MS
+    dask_client : dask.client
+        Dask client
     polcal_fields : list
         Polcal fields
     polcal_scans : list
         Polcal scans
     workdir : str
         Work directory
-    n_threads : int, optional
-        Number of OpenMP threads
-    dask_addr : str, optional
-        Dask scheduler address
+    cpu_frac : float, optional
+        CPU fraction to use
 
     Returns
     -------
@@ -346,6 +308,9 @@ def import_polcal_models(
         Success message
     """
     try:
+        if cpu_frac > 0.8:
+            cpu_frac = 0.8
+        total_cpu = max(1, int(psutil.cpu_count() * cpu_frac))
         if len(polcal_scans) == 0:
             print("No polarization calibrator scans is present.")
             return 1
@@ -354,30 +319,14 @@ def import_polcal_models(
             ms_scans = get_ms_scans(ms)
             for s in ms_scans:
                 scans.append(s)
-        mem_limit = polcal_setjy(dry_run=True)
-        if dask_addr is None:
-            dask_client, dask_cluster, n_jobs, n_threads, mem_limit, dask_dir = (
-                get_dask_client(
-                    len(polcal_scans),
-                    dask_dir=workdir,
-                    cpu_frac=cpu_frac,
-                    mem_frac=mem_frac,
-                    min_mem_per_job=mem_limit,
-                )
-            )
-        else:
-            _, _, n_jobs, n_threads, mem_limit, dask_dir = get_dask_client(
-                len(polcal_scans),
-                dask_dir=workdir,
-                cpu_frac=cpu_frac,
-                mem_frac=mem_frac,
-                min_mem_per_job=mem_limit,
-                only_cal=True,
-            )
-            os.system(f"rm -rf {dask_dir}")
-            dask_client = Client(address=dask_addr)
-        wait_for_dask_workers(dask_client,min_worker=1,timeout=60)
         tasks = []
+        total_scans = 0
+        for polcal_field in polcal_fields:
+            p_scan = polcal_scans[polcal_field]
+            for s in p_scan:
+                if s in scans:
+                    total_scans += 1
+        n_threads = max(1, int(total_cpu / total_scans))
         for polcal_field in polcal_fields:
             p_scan = polcal_scans[polcal_field]
             for s in p_scan:
@@ -389,19 +338,16 @@ def import_polcal_models(
                             sub_msname, polcal_field, ismms=True, n_threads=n_threads
                         )
                     )
-        futures = dask_client.compute(tasks)
-        results = list(dask_client.gather(futures))
-        dask_client.close()
-        if dask_addr is None:
-            dask_cluster.close()
-            os.system(f"rm -rf {dask_dir}")
+        print("Start importing polcal models...")
+        results = list(dask_client.gather(dask_client.compute(tasks)))
+        print("Polcal models imports are done.")
         return 0
     except Exception as e:
         traceback.print_exc()
         return 1
 
 
-def import_all_models(msname, workdir, cpu_frac=0.8, mem_frac=0.8, dask_addr=None):
+def import_all_models(msname, dask_client, workdir, cpu_frac=0.8, mem_frac=0.8):
     """
     Import all calibrator models
 
@@ -409,21 +355,20 @@ def import_all_models(msname, workdir, cpu_frac=0.8, mem_frac=0.8, dask_addr=Non
     ----------
     msname : str
         Measurement set name
+    dask_client : dask.client
+        Dask client
     workdir : str
         Work directory
     cpu_frac : float, optional
         CPU fraction to use
     mem_frac : float, optional
         Memory fraction to use
-    dask_addr : str, optional
-        Dask scheduler address
 
     Returns
     -------
     int
         Success message
     """
-    start_time = time.time()
     cpu_threads = psutil.cpu_count()
     ncpu = int(cpu_threads * cpu_frac)
     mem_frac = 1 - mem_frac
@@ -445,44 +390,37 @@ def import_all_models(msname, workdir, cpu_frac=0.8, mem_frac=0.8, dask_addr=Non
             mslist, fluxcal_fields, fluxcal_scans, ncpus=ncpu, mem_frac=mem_frac
         )
         if fluxcal_result != 0:
-            print("##################")
-            print("Total time taken : " + str(time.time() - start_time) + "s")
-            print("##################\n")
             return fluxcal_result, 1, 1
+
         phasecal_result = import_phasecal_models(
             mslist,
+            dask_client,
             phasecal_fields,
             phasecal_scans,
             workdir,
             cpu_frac=cpu_frac,
-            mem_frac=mem_frac,
-            dask_addr=dask_addr,
         )
+
         polcal_result = import_polcal_models(
             mslist,
+            dask_client,
             polcal_fields,
             polcal_scans,
             workdir,
             cpu_frac=cpu_frac,
-            mem_frac=mem_frac,
-            dask_addr=dask_addr,
         )
+
         if phasecal_result != 0:
             print(
                 "Phasecal model was not import, but fluxcal model import is successful."
             )
+
         if polcal_result != 0:
             print(
                 "Polcal model was not imported, but fluxcal model import is successful."
             )
-        print("##################")
-        print("Total time taken : " + str(time.time() - start_time) + "s")
-        print("##################\n")
         return fluxcal_result, phasecal_result, polcal_result
     except Exception as e:
-        print("##################")
-        print("Total time taken : " + str(time.time() - start_time) + "s")
-        print("##################\n")
         traceback.print_exc()
         return 1, 1, 1
 
@@ -495,7 +433,7 @@ def main(
     logfile=None,
     jobid=0,
     start_remote_log=False,
-    dask_addr=None,
+    dask_client=None,
 ):
     """
     Run the calibrator model import pipeline.
@@ -518,8 +456,8 @@ def main(
     start_remote_log : bool, optional
         If True, enables remote logging based on credentials stored in the
         working directory. Default is False.
-    dask_addr : str, optional
-        Dask scheduler address
+    dask_client : dask.client, optional
+        Dask client
 
     Returns
     -------
@@ -551,16 +489,26 @@ def main(
             observer = init_logger(
                 "import_model", logfile, jobname=jobname, password=password
             )
-    ###########
+
+    dask_cluster = None
+    if dask_client is None:
+        dask_client, dask_cluster, dask_dir = get_local_dask_cluster(
+            2,
+            dask_dir=workdir,
+            cpu_frac=cpu_frac,
+            mem_frac=mem_frac,
+        )
+        nworker = max(2, int(psutil.cpu_count() * cpu_frac))
+        scale_worker_and_wait(dask_cluster, nworker)
 
     try:
         if msname and os.path.exists(msname):
             fluxcal_result, phasecal_result, polcal_result = import_all_models(
                 os.path.abspath(msname),
+                dask_client,
                 workdir,
                 cpu_frac=cpu_frac,
                 mem_frac=mem_frac,
-                dask_addr=dask_addr,
             )
             os.system("touch " + workdir + "/.fluxcal_" + str(fluxcal_result))
             os.system("touch " + workdir + "/.phasecal_" + str(phasecal_result))
@@ -572,7 +520,7 @@ def main(
             else:
                 msg = 0
         else:
-            print("Please provide correct measurement set.\n")
+            print("Please provide correct measurement set.")
             msg = 1
     except Exception as e:
         traceback.print_exc()
@@ -585,6 +533,10 @@ def main(
         drop_cache(msname)
         drop_cache(workdir)
         clean_shutdown(observer)
+        if dask_cluster is not None:
+            dask_client.close()
+            dask_cluster.close()
+            os.system(f"rm -rf {dask_dir}")
     return msg
 
 
@@ -626,7 +578,7 @@ def cli():
 
     if len(sys.argv) == 1:
         parser.print_help(sys.stderr)
-        sys.exit(1)
+        return 1
 
     args = parser.parse_args()
 
