@@ -13,7 +13,6 @@ from astropy.io import fits
 from astropy.time import Time
 from astropy.coordinates import EarthLocation, SkyCoord, AltAz
 from astropy.wcs import FITSFixedWarning
-from casatasks import casalog
 from numpy.linalg import inv
 from astropy.wcs import WCS
 from scipy.interpolate import RectBivariateSpline
@@ -25,14 +24,8 @@ from meersolar.utils import (
     save_pid,
 )
 
-logging.getLogger("distributed").setLevel(logging.WARNING)
-
-try:
-    casalogfile = casalog.logfile()
-    os.system("rm -rf " + casalogfile)
-except BaseException:
-    pass
-
+logging.getLogger("distributed").setLevel(logging.ERROR)
+logging.getLogger("tornado.application").setLevel(logging.CRITICAL)
 datadir = get_datadir()
 
 warnings.simplefilter("ignore", category=FITSFixedWarning)
@@ -446,7 +439,6 @@ def get_image_beam(
     """
     if n_cpu > 8:
         n_cpu = 8
-    start_time = time.time()
     ##################################
     header = fits.getheader(image_file)
     if header["CTYPE3"] == "FREQ":
@@ -496,7 +488,7 @@ def get_image_beam(
         m_grid_flat = m_grid.flatten()
         grid_shape = l_grid.shape
         del l_grid, m_grid
-        with Parallel(n_jobs=n_cpu, backend="threading") as parallel:
+        with Parallel(njobs=n_cpu, backend="threading") as parallel:
             results = parallel(
                 [
                     joblid_delayed(j00_r)(l_grid_flat, m_grid_flat, grid=False),
@@ -555,8 +547,6 @@ def get_image_beam(
         # matrix
         jones_array = apply_parallactic_rotation(jones_array, p_angle).T
     jones_array = jones_array.reshape(jones_array.shape[0], jones_array.shape[1], 2, 2)
-    if verbose:
-        print(f"Beam calculated in : {round(time.time()-start_time,1)}s")
     return jones_array
 
 
@@ -794,7 +784,7 @@ def cli():
 
     if len(sys.argv) == 1:
         parser.print_help(sys.stderr)
-        sys.exit(1)
+        return 1
 
     args = parser.parse_args()
 
