@@ -15,4 +15,12 @@ from .sunpos_utils import *
 from .udocker_utils import *
 from .prefect_logger_utils import *
 from .prefect_setup_utils import *
+import os
+from casatasks import casalog
 
+set_udocker_env()
+try:
+    logfile = casalog.logfile()
+    os.remove(logfile)
+except BaseException:
+    pass
