@@ -219,7 +219,7 @@ def get_valid_scans(msname, field="", min_scan_time=1, n_threads=-1):
         msmd = msmetadata()
         msmd.open(msname)
         for f in field:
-            with suppress_casa_output():
+            with suppress_output():
                 try:
                     field_id = msmd.fieldsforname(f)[0]
                 except Exception as e:
@@ -505,7 +505,6 @@ def split_noise_diode_scans(
     scan="",
     datacolumn="data",
     n_threads=-1,
-    dry_run=True,
 ):
     """
     Split noise diode on and off timestamps into two seperate measurement sets
@@ -533,10 +532,6 @@ def split_noise_diode_scans(
     limit_threads(n_threads=n_threads)
     from casatasks import split
 
-    if dry_run:
-        process = psutil.Process(os.getpid())
-        mem = round(process.memory_info().rss / 1024**3, 2)  # in GB
-        return mem
     msname = msname.rstrip("/")
     mspath = os.path.dirname(os.path.abspath(msname))
     os.chdir(mspath)
