@@ -324,7 +324,7 @@ def check_datacolumn_valid(msname, datacolumn="DATA"):
         return False
 
 
-def get_bad_ants(msname="", fieldnames=[], n_threads=-1, dry_run=False):
+def get_bad_ants(msname="", fieldnames=[], n_threads=-1):
     """
     Get bad antennas
 
@@ -348,10 +348,6 @@ def get_bad_ants(msname="", fieldnames=[], n_threads=-1, dry_run=False):
     if len(fieldnames) == 0:
         print("Provide field names.")
         return [], ""
-    if dry_run:
-        process = psutil.Process(os.getpid())
-        mem = round(process.memory_info().rss / 1024**3, 2)  # in GB
-        return mem
     msname = msname.rstrip("/")
     mspath = os.path.dirname(os.path.abspath(msname))
     os.chdir(mspath)
@@ -441,7 +437,7 @@ def get_common_spw(spw1, spw2):
     return to_str(to_set(spw1) & to_set(spw2))
 
 
-def scans_in_timerange(msname="", timerange="", dry_run=False):
+def scans_in_timerange(msname="", timerange=""):
     """
     Get scans in the given timerange
 
@@ -459,10 +455,6 @@ def scans_in_timerange(msname="", timerange="", dry_run=False):
     """
     from casatools import ms, quanta
 
-    if dry_run:
-        process = psutil.Process(os.getpid())
-        mem = round(process.memory_info().rss / 1024**3, 2)  # in GB
-        return mem
     msname = msname.rstrip("/")
     mspath = os.path.dirname(os.path.abspath(msname))
     os.chdir(mspath)
@@ -511,7 +503,11 @@ def scans_in_timerange(msname="", timerange="", dry_run=False):
     return valid_scans
 
 
-def get_refant(msname="", field="", n_threads=-1, dry_run=False):
+def get_refant(
+    msname="",
+    field="",
+    n_threads=-1,
+):
     """
     Get reference antenna
 
@@ -530,10 +526,6 @@ def get_refant(msname="", field="", n_threads=-1, dry_run=False):
     limit_threads(n_threads=n_threads)
     from casatasks import visstat, casalog
 
-    if dry_run:
-        process = psutil.Process(os.getpid())
-        mem = round(process.memory_info().rss / 1024**3, 2)  # in GB
-        return mem
     msname = msname.rstrip("/")
     mspath = os.path.dirname(os.path.abspath(msname))
     os.chdir(mspath)
@@ -580,6 +572,49 @@ def get_refant(msname="", field="", n_threads=-1, dry_run=False):
     return str(referenceant)
 
 
+def get_uvrange_exclude(uvrange):
+    """
+    Get uv-range(s) excluding the given uv-range
+
+    Parameters
+    ----------
+    uvrange : str
+        UV-range in CASA format
+
+    Returns
+    -------
+    list
+        List of uvranges excluding the given uv-range
+    """
+    uvrange = uvrange.strip().lower()
+    if "lambda" not in uvrange:
+        raise ValueError("uvrange must contain 'lambda' units")
+    if uvrange.startswith(">"):
+        val = uvrange[1:].replace("lambda", "").strip()
+        return [f"<{val}lambda"]
+    elif uvrange.startswith("<"):
+        val = uvrange[1:].replace("lambda", "").strip()
+        return [f">{val}lambda"]
+    elif "~" in uvrange:
+        parts = uvrange.replace("lambda", "").split("~")
+        if len(parts) != 2:
+            raise ValueError("Invalid uvrange format with '~'")
+        low, high = parts[0].strip(), parts[1].strip()
+        try:
+            low_val = float(low)
+            high_val = float(high)
+        except ValueError:
+            raise ValueError("uvrange bounds must be numeric")
+        if low_val > high_val:
+            raise ValueError(
+                f"Lower bound {low_val} > upper bound {high_val} in uvrange"
+            )
+        return [f"<{low}lambda", f">{high}lambda"]
+
+    else:
+        raise ValueError(f"Unsupported uvrange format: '{uvrange}'")
+
+
 def get_ms_scans(msname):
     """
     Get scans of the measurement set
